'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deferURL = deferURL;
exports.launch = launch;
exports.launchNewNotebook = launchNewNotebook;
exports.launchFilename = launchFilename;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _electron = require('electron');

var _commutable = require('commutable');

var _immutable = require('immutable');

var immutable = _interopRequireWildcard(_immutable);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = require('electron-log');

function deferURL(event, url) {
  event.preventDefault();
  _electron.shell.openExternal(url);
}

function launch(notebook, filename) {
  let iconPath = '';
  if (process.argv[0].match(/electron/i)) {
    iconPath = './static/icon.png';
  } else {
    iconPath = '.cd/resources/app/static/icon.png';
  }

  let win = new _electron.BrowserWindow({
    width: 800,
    height: 1000,
    title: !filename ? 'Untitled' : _path2.default.relative('.', filename.replace(/.ipynb$/, '')),
    icon: iconPath
  });

  const index = _path2.default.join(__dirname, '..', '..', 'static', 'index.html');
  win.loadURL(`file://${ index }`);

  // When the page finishes loading, send the notebook data via IPC
  win.webContents.on('did-finish-load', () => {
    win.webContents.send('main:load', { notebook: notebook.toJS(), filename });
  });

  win.webContents.on('will-navigate', deferURL);

  // Emitted when the window is closed.
  win.on('closed', () => {
    win = null;
  });
  return win;
}

function launchNewNotebook(kernelspec) {
  const starterNotebook = (0, _commutable.appendCell)(_commutable.emptyNotebook, _commutable.emptyCodeCell);

  let nb = starterNotebook;

  if (kernelspec && kernelspec.name && kernelspec.spec && kernelspec.spec.language && kernelspec.spec.display_name) {
    nb = starterNotebook.setIn(['metadata', 'kernelspec'], immutable.fromJS({
      name: kernelspec.name,
      language: kernelspec.spec.language,
      display_name: kernelspec.spec.display_name
    }));
  }

  return Promise.resolve(launch(nb));
}

function launchFilename(filename) {
  if (!filename) {
    const warning = 'WARNING: launching a notebook with no filename and no kernelspec';
    log.info(warning); // Since it's not really a warning, this is our default case
    console.warn(warning);
    return Promise.resolve(launchNewNotebook());
  }

  return new Promise((resolve, reject) => {
    _fs2.default.readFile(filename, {}, (err, data) => {
      if (err) {
        reject(err);
        const warning = 'Filename not resolved, launching an empty Notebook.';
        // TODO: This should open a new notebook with the given filename
        log.warn(warning);
        console.warn(warning);
        launchNewNotebook('python3');
      } else {
        resolve(launch((0, _commutable.fromJS)(JSON.parse(data)), filename));
      }
    });
  });
}