'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tildify = tildify;
exports.initNativeHandlers = initNativeHandlers;
exports.launchFilename = launchFilename;

var _electron = require('electron');

var _homeDir = require('home-dir');

var _homeDir2 = _interopRequireDefault(_homeDir);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _commutable = require('commutable');

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _launch = require('../main/launch');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const HOME = (0, _homeDir2.default)();
const { BrowserWindow, getCurrentWindow } = _electron.remote;

/**
 * Turn a path like /Users/n/mine.ipynb to ~/mine.ipynb
 * @param  {string} p the full path to a file
 * @return {string}   tildified path
 */
function tildify(p) {
  if (!p) {
    return '';
  }
  const s = _path2.default.normalize(p) + _path2.default.sep;
  return (s.indexOf(HOME) === 0 ? s.replace(HOME + _path2.default.sep, `~${ _path2.default.sep }`) : s).slice(0, -1);
}

function initNativeHandlers(store) {
  _Rx2.default.Observable.from(store).map(state => {
    const modified = state.app.get('modified');
    const executionState = state.app.get('executionState');
    const filename = tildify(state.metadata.get('filename')) || 'Untitled';
    const displayName = state.document.getIn(['notebook', 'metadata', 'kernelspec', 'display_name'], '...');

    return {
      title: `${ filename } - ${ displayName } - ${ executionState } ${ modified ? '*' : '' }`,
      path: filename
    };
  }).distinctUntilChanged().debounceTime(200).subscribe(res => {
    const win = getCurrentWindow();
    // TODO: Investigate if setRepresentedFilename() is a no-op on non-OS X
    if (res.path && win.setRepresentedFilename) {
      // TODO: this needs to be the full path to the file
      win.setRepresentedFilename(res.path);
    }
    win.setTitle(res.title);
  });
}

function launch(notebook, filename) {
  // TODO: This code is also done in main and we need to make sure it's consistent
  //       Rely on sourcing from ../main/{something}
  let win = new BrowserWindow({
    width: 800,
    height: 1000,
    title: !filename ? 'Untitled' : _path2.default.relative('.', filename.replace(/.ipynb$/, ''))
  });

  const index = _path2.default.join(__dirname, '..', '..', '..', 'static', 'index.html');
  win.loadURL(`file://${ index }`);

  win.webContents.on('did-finish-load', () => {
    win.webContents.send('main:load', { notebook: notebook.toJS(), filename });
  });

  win.webContents.on('will-navigate', _launch.deferURL);

  win.on('closed', () => {
    win = null;
  });

  return win;
}

function launchFilename(filename) {
  return new Promise((resolve, reject) => {
    _fs2.default.readFile(filename, {}, (err, data) => {
      if (err) {
        reject(err);
      } else {
        resolve(launch((0, _commutable.fromJS)(JSON.parse(data)), filename));
      }
    });
  });
}